function dynare_estimation
  global options_ bayestopt_ estim_params_ lgy_ lgx_ Sigma_e_ exo_nbr endo_nbr
  global lgy_ lgx_ fname_ oo_  trend_coeff_
  
  options_ = set_default_option(options_,'first_obs',1);
  options_ = set_default_option(options_,'prefilter',0);
  options_ = set_default_option(options_,'presample',0);
  options_ = set_default_option(options_,'lik_algo',1);
  options_ = set_default_option(options_,'lik_init',1);
  options_ = set_default_option(options_,'nograph',0);
  options_ = set_default_option(options_,'mh_conf_sig',0.90);
  options_ = set_default_option(options_,'mh_replic',20000);
  options_ = set_default_option(options_,'mh_drop',0.5);
  options_ = set_default_option(options_,'mh_jscale',0.2);
  options_ = set_default_option(options_,'mh_init_scale',2*options_.mh_jscale);
  options_ = set_default_option(options_,'mode_file','');
  options_ = set_default_option(options_,'mode_compute',4);
  options_ = set_default_option(options_,'mode_check',0);
  options_ = set_default_option(options_,'prior_trunc',1e-10);
  options_ = set_default_option(options_,'mh_mode',1); 	
  options_ = set_default_option(options_,'mh_nblck',2);	
  options_ = set_default_option(options_,'load_mh_file',0);
  options_ = set_default_option(options_,'nodiagnostic',0);
  options_ = set_default_option(options_,'loglinear',0);
  
  optim_options = optimset('display','iter','LargeScale','off', ...
			   'MaxFunEvals',100000,'TolFun',1e-8,'TolX',1e-6);
  if isfield(options_,'optim_opt')
    eval(['optim_options = optimset(optim_options,' options_.optim_opt ');']);
  end

  pnames=['     ';'beta ';'gamm ';'norm ';'invg ';'unif ';'invg2'];
  
  n_varobs = size(options_.varobs,1);
  [xparam1,estim_params_,bayestopt_,lb,ub]=set_prior(estim_params_);
  if any(bayestopt_.pshape > 0)
    compdist(xparam1,[],[],'Priors');
  end
  bounds = prior_bounds(bayestopt_);
  bounds(:,1)=max(bounds(:,1),lb);
  bounds(:,2)=min(bounds(:,2),ub);
  if any(xparam1 < bounds(:,1)) | any(xparam1 > bounds(:,2))
    find(xparam1 < bounds(:,1))
    find(xparam1 > bounds(:,2))
    error('Initial parameter values are outside parameter bounds')
  end
  lb = bounds(:,1);
  ub = bounds(:,2);
  bayestopt_.lb = lb;
  bayestopt_.ub = ub;

  if isempty(trend_coeff_)
    bayestopt_.with_trend = 0;
  else
    bayestopt_.with_trend = 1;
    bayestopt_.trend_coeff = {};
    for i=1:n_varobs
      if i > length(trend_coeff_) | isempty(trend_coeff_{i})
	bayestopt_.trend_coeff{i} = '0';
      else
	bayestopt_.trend_coeff{i} = trend_coeff_{i};
      end
    end
  end
    
  bayestopt_.penalty=1e8;             % penalty 


  nvx = estim_params_.nvx;
  nvn = estim_params_.nvn;
  ncx = estim_params_.ncx;
  ncn = estim_params_.ncn;
  np  = estim_params_.np ;
  nx = nvx+nvn+ncx+ncn+np;
  
  dr = set_state_space([]);
  
  if isempty(options_.datafile)
    error('ESTIMATION: datafile option is missing')
  end
  
  if isempty(options_.varobs)
    error('ESTIMATION: VAROBS is missing')
  end
  

  % if jscale isn't specified for an estimated parameter, use
  % global option options_.jscale, set to 0.2, by default
  k = find(isnan(bayestopt_.jscale));
  bayestopt_.jscale(k) = options_.mh_jscale;

  % read and demean data 
  if exist(options_.datafile)
    instr = options_.datafile;
  else
    instr = ['load ' options_.datafile];
  end
  eval(instr);

  rawdata = [];
  k = [];
  k1 = [];
  for i=1:n_varobs
    rawdata = [rawdata eval(deblank(options_.varobs(i,:)))];
    k = [k strmatch(deblank(options_.varobs(i,:)),lgy_(dr.order_var,:), ...
		    'exact')];
    k1 = [k1 strmatch(deblank(options_.varobs(i,:)),lgy_, 'exact')];
  end
  
  bayestopt_.mf = k;
  bayestopt_.mfys = k1;

  options_ = set_default_option(options_,'nobs',size(rawdata,1)-options_.first_obs+1);
  gend = options_.nobs;
  rawdata = rawdata(options_.first_obs:options_.first_obs+gend-1,:);
  if options_.loglinear == 1
    rawdata = log(rawdata);
  end
  if options_.prefilter == 1
    mean_varobs = mean(rawdata,1);
    data = rawdata-ones(gend,1)*mean_varobs;
  else
    data = rawdata;
  end

  if ~isreal(rawdata)
    error(['There are complex values in the data. Probably  a wrong' ...
	   ' transformation'])
  end
  
  if length(options_.mode_file) > 0
    eval(['load ' options_.mode_file ';']');
  end
  
  initial_estimation_checks(xparam1,gend,data);
  
  if options_.mode_compute > 0
    fh=str2func('mj_optmumlik');
    if options_.mode_compute == 1  
      [xparam1,fval,exitflag,output,lamdba,grad,hessian_fmincon] = ...
	  fmincon(fh,xparam1,[],[],[],[],lb,ub,[],optim_options,gend,data,1);
    elseif options_.mode_compute == 2
      %	asamin('set','maximum_cost_repeat',0);
      [fval,xparam1,grad,hessian_asamin,exitflag] = ...
	  asamin('minimize','mj_optmumlik',xparam1,lb,ub,- ...
		   ones(size(xparam1)),gend,data,1);   
    elseif options_.mode_compute == 3
      [xparam1,fval,exitflag] = fminunc(fh,xparam1,optim_options,gend, ...
					data,1);
    elseif options_.mode_compute == 4
      H0 = 1e-4*eye(nx);
      crit = 1e-7;
      nit = 1000;
      verbose = 2;
      [fval,xparam1,grad,hessian_csminwel,itct,fcount,retcodehat] = ...
	  csminwel('mj_optmumlik',xparam1,H0,[],crit,nit,gend,data,1);
      disp(sprintf('Objective function at mode: %f',fval))
      disp(sprintf('Objective function at mode: %f',mj_optmumlik(xparam1,gend,data,1)))
    end
    hh = reshape(hessian('mj_optmumlik',xparam1,gend,data,1),nx,nx);
    eval(['save ' fname_ '_mode xparam1 hh;']);
  end

  if options_.mode_check == 1
    mode_check(xparam1,0,hh,gend,data,lb,ub);
  end
  
  hh = generalized_cholesky(hh);
  invhess = inv(hh);
  stdh = sqrt(diag(invhess));
  if any(bayestopt_.pshape > 0)
    disp(' ')
    disp('RESULTS FROM POSTERIOR MAXIMIZATION')
    tstath = zeros(nx,1);
    for i = 1:nx
      tstath(i) = abs(xparam1(i))/stdh(i);
    end
    tit1 = sprintf('%10s %7s %8s %7s %6s %4s %6s\n',' ','prior mean', ...
		   'mode','s.d.','t-stat','prior','pstdev');
    if np
      ip = nvx+nvn+ncx+ncn+1;
      disp('parameters')
      disp(tit1)
      for i=1:np
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', ...
		     deblank(estim_params_.param_names(i,:)), ...
		     bayestopt_.pmean(ip),xparam1(ip),stdh(ip),tstath(ip), ...
		     pnames(bayestopt_.pshape(ip)+1,:), ...
		     bayestopt_.pstdev(ip))); 
	ip = ip+1;
      end
    end
    if nvx
      ip = 1;
      disp('standard deviation of shocks')
      disp(tit1)
      for i=1:nvx
	k = estim_params_.var_exo(i,1);
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', ...
		     deblank(lgx_(k,:)),bayestopt_.pmean(ip),xparam1(ip), ...
		     stdh(ip),tstath(ip),pnames(bayestopt_.pshape(ip)+1,:), ...
		     bayestopt_.pstdev(ip))); 
	Sigma_e_(k,k) = xparam1(ip)*xparam1(ip);
	ip = ip+1;
      end
    end
    if nvn
      disp('standard deviation of measurement errors')
      disp(tit1)
      ip = nvx+1;
      for i=1:nvn
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', ...
		     deblank(options_.varobs(estim_params_.var_endo(i,1),: ...
					     )),bayestopt_.pmean(ip), ...
		     xparam1(ip),stdh(ip),tstath(ip), ...
		     pnames(bayestopt_.pshape(ip)+1,:), ...
		     bayestopt_.pstdev(ip))); 
	ip = ip+1;
      end
    end
    if ncx
      disp('correlation of shocks')
      disp(tit1)
      ip = nvx+nvn+1;
      for i=1:ncx
	k1 = estim_params_.corrx(i,1);
	k2 = estim_params_.corrx(i,2);
	name = [deblank(lgx_(k1,:)) ',' deblank(lgx_(k2,:))];
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', name, ...
		     bayestopt_.pmean(ip),xparam1(ip),stdh(ip),tstath(ip),  ...
		     pnames(bayestopt_.pshape(ip)+1,:), bayestopt_.pstdev(ip)));
	Sigma_e_(k1,k2) = xparam1(ip)*sqrt(Sigma_e_(k1,k1)*Sigma_e_(k2,k2));
	Sigma_e_(k2,k1) = Sigma_e_(k1,k2);
	ip = ip+1;
      end
    end
    if ncn
      disp('correlation of measurement errors')
      disp(tit1)
      ip = nvx+nvn+ncx+1;
      for i=1:ncn
	k1 = estim_params_.corrn(i,1);
	k2 = estim_params_.corrn(i,2);
	name = [deblank(lgx_(k1,:)) ',' deblank(lgx_(k2,:))];
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', name, ...
		     bayestopt_.pmean(ip),xparam1(ip),stdh(ip),tstath(ip), ...
		     pnames(bayestopt_.pshape(ip)+1,:), bayestopt_.pstdev(ip)));
	ip = ip+1;
      end
    end
    %%% Laplace approximation to the marginal log density: 
    md_Laplace = .5*size(xparam1,1)*log(2*pi) + .5*log(det(invhess)) ...
        - mj_optmumlik(xparam1,gend,data,1);
    disp(' ')
    disp(sprintf('Log data density [Laplace approximation] is %f.',md_Laplace))
    disp(' ')
  else
    disp(' ')
    disp('RESULTS FROM MAXIMUM LIKELIHOOD')
    tstath = zeros(nx,1);
    for i = 1:nx
      tstath(i) = abs(xparam1(i))/stdh(i);
    end
    tit1 = sprintf('%10s %10s %7s %6s\n',' ', ...
		   'Estimate','s.d.','t-stat');
    if np
      ip = nvx+nvn+ncx+ncn+1;
      disp('parameters')
      disp(tit1)
      for i=1:np
	disp(sprintf('%12s %8.4f %7.4f %7.4f', ...
		     deblank(estim_params_.param_names(i,:)), ...
		     xparam1(ip),stdh(ip),tstath(ip)));
	ip = ip+1;
      end
    end
    if nvx
      ip = 1;
      disp('standard deviation of shocks')
      disp(tit1)
      for i=1:nvx
	k = estim_params_.var_exo(i,1);
	disp(sprintf('%12s %8.4f %7.4f %7.4f', ...
		     deblank(lgx_(k,:)),xparam1(ip), ...
		     stdh(ip),tstath(ip)));
	Sigma_e_(k,k) = xparam1(ip)*xparam1(ip);
	ip = ip+1;
      end
    end
    if nvn
      disp('standard deviation of measurement errors')
      disp(tit1)
      ip = nvx+1;
      for i=1:nvn
	disp(sprintf('%12s %8.4f %7.4f %7.4f', ...
		     deblank(options_.varobs(estim_params_.var_endo(i,1),: ...
					     )), ...
		     xparam1(ip),stdh(ip),tstath(ip)))
	ip = ip+1;
      end
    end
    if ncx
      disp('correlation of shocks')
      disp(tit1)
      ip = nvx+nvn+1;
      for i=1:ncx
	k1 = estim_params_.corrx(i,1);
	k2 = estim_params_.corrx(i,2);
	name = [deblank(lgx_(k1,:)) ',' deblank(lgx_(k2,:))];
	disp(sprintf('%12s %8.4f %7.4f %7.4f', name, ...
		     xparam1(ip),stdh(ip),tstath(ip)));
	Sigma_e_(k1,k2) = xparam1(ip)*sqrt(Sigma_e_(k1,k1)*Sigma_e_(k2,k2));
	Sigma_e_(k2,k1) = Sigma_e_(k1,k2);
	ip = ip+1;
      end
    end
    if ncn
      disp('correlation of measurement errors')
      disp(tit1)
      ip = nvx+nvn+ncx+1;
      for i=1:ncn
	k1 = estim_params_.corrn(i,1);
	k2 = estim_params_.corrn(i,2);
	name = [deblank(lgx_(k1,:)) ',' deblank(lgx_(k2,:))];
	disp(sprintf('%12s %8.4f %7.4f %7.4f', name, ...
		     xparam1(ip),stdh(ip),tstath(ip)));
	ip = ip+1;
      end
    end
  end
  
  if options_.load_mh_file == 1
    files = eval(['dir(''' fname_ '_mh*.mat'');']);
    if length(files) == 0
      error('LOAD_MH_FILE: there is no MH file to load here')
    end
    kmax = 0;
    for i=1:length(files)
      k = sscanf(files(i).name,[fname_ '_mh%d.mat']);
      if k > kmax
	kmax = k;
      end
    end
    fname1 = [fname_ '_mh' int2str(kmax)];
    eval(['load ' fname1]);
    fname2 = [fname_ '_mh' int2str(kmax+1)];
  else
    x2 = [];
    logpo2 = [];
    fname2 = [fname_ '_mh0'];
  end

  if options_.mh_replic > 0 & any(bayestopt_.pshape ~= 0) % not ML estimation
    bounds = prior_bounds(bayestopt_);
    if any(xparam1 < bounds(:,1)) | any(xparam1 > bounds(:,2))
      find(xparam1 < bounds(:,1))
      find(xparam1 > bounds(:,2))
      error('Mode values are outside prior bounds. Reduce prior_trunc.')
    end
    
    
    [x2, logpo2] = metropolis(xparam1, invhess, gend, data, bounds, fname2, ...
			      options_.mh_mode,x2,logpo2);

  end
  
  if ~isempty(x2)
    if options_.nodiagnostic == 0
      mcmc_diagnostic(x2);
    end
    
    md = marginal_density(x2,logpo2)

    x2 = x2(ceil(options_.mh_drop*size(x2,1))+1:end,:,:);
    tmp1 = []; 
    for b=1:size(x2,3);
        tmp1 = [tmp1 ; x2(:,:,b)];        
    end;
    x2 = tmp1;
    clear tmp1;

    disp(' ')
    disp('ESTIMATION RESULTS')
    disp(' ')
    disp(sprintf('Log data density is %f.',mean(md(:,2))))
    
    [post_mean, min_interval] = posterior_moments(x2);

    tit2 = sprintf('%10s %7s %10s %14s %4s %6s\n',' ','prior mean', ...
		   'post. mean','conf. interval','prior','pstdev');
    ip = nvx+nvn+ncx+ncn+1;
    if np
      disp('parameters')
      disp(tit2)
      for i=1:np
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', ...
		     deblank(estim_params_.param_names(i,:)), ...
		     bayestopt_.pmean(ip),post_mean(ip),min_interval(ip,:), ...
		     pnames(bayestopt_.pshape(ip)+1,:), ...
		     bayestopt_.pstdev(ip))); 
	ip = ip+1;
      end
    end
    if nvx
      ip = 1;
      disp('standard deviation of shocks')
      disp(tit2)
      for i=1:nvx
	k = estim_params_.var_exo(i,1);
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', ...
		     deblank(lgx_(k,:)),bayestopt_.pmean(ip),post_mean(ip), ...
		     min_interval(ip,:),pnames(bayestopt_.pshape(ip)+1,:), ...
		     bayestopt_.pstdev(ip))); 
	Sigma_e_(k,k) = post_mean(ip)*post_mean(ip);
	ip = ip+1;
      end
    end
    if nvn
      disp('standard deviation of measurement errors')
      disp(tit2)
      ip = nvx+1;
      for i=1:nvn
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', ...
		     deblank(options_.varobs(estim_params_.var_endo(i,1), ...
					     :)),bayestopt_.pmean(ip), ...
		     post_mean(ip),min_interval(ip,:), ...
		     pnames(bayestopt_.pshape(ip)+1,:), ...
		     bayestopt_.pstdev(ip))); 
	ip = ip+1;
      end
    end
    if ncx
      disp('correlation of shocks')
      disp(tit2)
      ip = nvx+nvn+1;
      for i=1:ncx
	k1 = estim_params_.corrx(i,1);
	k2 = estim_params_.corrx(i,2);
	name = [deblank(lgx_(k1,:)) ',' deblank(lgx_(k2,:))];
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', name, ...
		     bayestopt_.pmean(ip),post_mean(ip),min_interval(ip,:), ...
		     pnames(bayestopt_.pshape(ip)+1,:), ...
		     bayestopt_.pstdev(ip))); 
	Sigma_e_(k1,k2) = post_mean(ip)*sqrt(Sigma_e_(k1,k1)*Sigma_e_(k2,k2));
	Sigma_e_(k2,k1) = Sigma_e_(k1,k2);
	ip = ip+1;
      end
    end
    if ncn
      disp('correlation of measurement errors')
      disp(tit2)
      ip = nvx+nvn+ncx+1;
      for i=1:ncn
	k1 = estim_params_.corrn(i,1);
	k2 = estim_params_.corrn(i,2);
	name = [deblank(lgx_(k1,:)) ',' deblank(lgx_(k2,:))];
	disp(sprintf('%12s %7.3f %8.4f %7.4f %7.4f %4s %6.4f', name, ...
		     bayestopt_.pmean(ip),post_mean(ip),min_interval(ip,:), ...
		     pnames(bayestopt_.pshape(ip)+1,:), ...
		     bayestopt_.pstdev(ip))); 
	ip = ip+1;
      end
    end      % end of estimation results for Bayesian estimation
    if options_.nograph == 0
      compdist(xparam1, x2, 'All','Priors and posteriors');
    end
    xparam1 = post_mean; % next section expects point estimates in xparam1
  end        % end of metropolis-hasting computation
  
  options_.lik_algo = 2;
  [lnprior,cost_flag,atT,innov,ys,trend_coeff] = ...
      mj_optmumlik(xparam1,gend,data,2);
  
  for i=1:endo_nbr
    eval(['oo_.' deblank(lgy_(dr.order_var(i),:)) '_smooth = atT(i,:)'';']);
  end
  
  if options_.nograph == 0
    if exo_nbr == 1
      nr = 1;
      nc = 1;
    elseif exo_nbr == 2
      nr = 1;
      nc = 2;
    elseif exo_nbr <= 4
      nr = 2;
      nc = 2;
    elseif exo_nbr <= 6
      nr = 2;
      nc = 3;
    elseif exo_nbr <= 9
      nr = 3;
      nc = 3;
    elseif exo_nbr <= 12
      nr = 3;
      nc = 4;
    end

    figure('Name','Smoothed shocks');
    for i=1:exo_nbr
      subplot(nr,nc,i);
      plot([innov(i,:)' zeros(size(innov(i,:)'))])
      title(lgx_(i,:),'Interpreter','none')
      eval(['oo_.' deblank(lgx_(i,:)) '_smooth = innov(i,:)'';']);
    end
    
    yf = zeros(gend,n_varobs);
    if options_.prefilter == 1
      yf = atT(bayestopt_.mf,:)+repmat(mean_varobs',1,gend+1);
    elseif options_.loglinear == 1
      yf = atT(bayestopt_.mf,:)+repmat(log(ys(bayestopt_.mfys)),1,gend+1)+...
	   trend_coeff*[0:gend];
    else
      yf = atT(bayestopt_.mf,:)+repmat(ys(bayestopt_.mfys),1,gend+1)+...
	   trend_coeff*[0:gend];
    end
    
    if nvn
      figure('Name','Smoothed observation errors');
      for i=1:n_varobs
	subplot(nr,nc,i);
	plot([rawdata(:,i)-yf(i,2:end)' zeros(size(rawdata(:,i)))]);
	title(options_.varobs(i,:),'Interpreter','none')
	eval(['oo_.' deblank(options_.varobs(i,:)) '_obs_err_smooth = rawdata(:,i)-yf(i,2:end)'';']);
      end
      
      if n_varobs == 1
	nr = 1;
	nc = 1;
      elseif n_varobs == 2
	nr = 1;
	nc = 2;
      elseif n_varobs <= 4
	nr = 2;
	nc = 2;
      elseif n_varobs <= 6
	nr = 2;
	nc = 3;
      elseif n_varobs <= 9
	nr = 3;
	nc = 3;
      elseif n_varobs <= 12
	nr = 3;
	nc = 4;
      end

      figure('Name','Historical and smoothed variables');
      for i=1:n_varobs
	subplot(nr,nc,i);
	plot([rawdata(:,i) yf(i,2:end)'])
	title(options_.varobs(i,:),'Interpreter','none')
      end
    end
  end % end if options_.nograph ~= 0
